<?php
/**
 * Caching for the Kolab free/busy data.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache.php,v 1.17.2.10 2010/06/15 12:00:33 wrobel Exp $
 *
 * @package Kolab_FreeBusy
 */

/** Load the cache components that we always need */
require_once 'Horde/Kolab/FreeBusy/Cache/File.php';
require_once 'Horde/Kolab/FreeBusy/Cache/File/pvcal.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB/acl.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB/xacl.php';

/**
 * The Horde_Kolab_FreeBusy_Cache:: class provides functionality to store
 * prepared free/busy data for quick retrieval.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache.php,v 1.17.2.10 2010/06/15 12:00:33 wrobel Exp $
 *
 * Copyright 2004-2008 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @author  Gunnar Wrobel <p@rdus.de>
 * @author  Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @package Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache {

    /**
     * The directory that should be used for caching.
     *
     * @var string
     */
    var $_cache_dir;

    /**
     * Constructor.
     *
     * @param string  $cache_dir  The cache directory we should use.
     */
    function Horde_Kolab_FreeBusy_Cache($cache_dir)
    {
        $this->_cache_dir = $cache_dir;
    }

    private function getPartials($access)
    {
        global $conf;

        require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl.php';
        require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl.php';
        require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Decorator/Log.php';
        if (!empty($conf['fb']['use_acls'])) {
            require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Cache.php';
            $xacldbcache = Horde_Kolab_FreeBusy_Cache_DB_xacl::singleton('xacl', $this->_cache_dir);
            require_once 'Horde/Kolab/FreeBusy/Cache/File/xacl.php';
            $xaclfilecache = new Horde_Kolab_FreeBusy_Cache_File_xacl($this->_cache_dir);
            $xacl_access = new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Decorator_Log(
                new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Cache(
                    $access, $xacldbcache, $xaclfilecache
                )
            );

            require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl/Cache.php';
            $acldbcache = &Horde_Kolab_FreeBusy_Cache_DB_acl::singleton('acl', $this->_cache_dir);
            require_once 'Horde/Kolab/FreeBusy/Cache/File/acl.php';
            $aclfilecache = new Horde_Kolab_FreeBusy_Cache_File_acl($this->_cache_dir);
            //@todo: log decorator that informs about the stored values.
            $acl_access = new Horde_Kolab_FreeBusy_Export_Freebusy_Acl_Cache(
                $access, $acldbcache, $aclfilecache
            );
        } else {
            require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Configuration.php';
            $xacl_access = new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Decorator_Log(
                new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Configuration(
                    $access, true
                )
            );
            require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl/Null.php';
            $acl_access = new Horde_Kolab_FreeBusy_Export_Freebusy_Acl_Null(
                $access, $this->_cache_dir
            );
        }

        require_once 'Horde/Kolab/FreeBusy/Cache/Freebusy/Partials.php';
        return new Horde_Kolab_FreeBusy_Cache_Freebusy_Partials(
            $this->_cache_dir, $acl_access, $xacl_access
        );
    }

    /**
     * Update the cache information for a calendar.
     *
     * @param Horde_Kolab_FreeBusy_Access $access The object holding the
     *                                      relevant access
     *                                      parameters.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function store($access)
    {
        global $conf;

        /* Now we really need the free/busy library */
        require_once 'Horde/Kolab/FreeBusy/Imap.php';

        $fb = new Horde_Kolab_FreeBusy_Imap();

        $result = $fb->connect($access->imap_folder);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $fbpast = $fbfuture = null;
        if (!empty($access->server_object)) {
            $result = $access->server_object->get(KOLAB_ATTR_FBPAST);
            if (!is_a($result, 'PEAR_Error')) {
                $fbpast = $result;
            }
        }
        if (!empty($access->owner_object)) {
            $result = $access->owner_object->get(KOLAB_ATTR_FBFUTURE);
            if (!is_a($result, 'PEAR_Error')) {
                $fbfuture = $result;
            }
        }

        $vCal = $fb->generate(null, null,
                              !empty($fbpast)?$fbpast:0,
                              !empty($fbfuture)?$fbfuture:60,
                              $access->owner,
                              $access->owner_object->get(KOLAB_ATTR_CN));
        if (is_a($vCal, 'PEAR_Error')) {
            return $vCal;
        }

        $fbfilename = $this->_getFilename($access->folder, $access->owner);
        if (empty($vCal)) {
            return $this->getPartials($access)->purgePartialVcal(
                $fbfilename
            );
        } else {
            return $this->getPartials($access)->storePartialVcal(
                $fbfilename,
                $vCal,
                $fb
            );
        }
    }

    /**
     * Load partial free/busy data.
     *
     * @param Horde_Kolab_FreeBusy_Access $access   The object holding the
     *                                        relevant access
     *                                        parameters.
     * @param boolean               $extended Should the data hold the extended
     *                                        free/busy information?
     *
     * @return Horde_iCalendar|PEAR_Error The free/busy data of a
     *                                    single calendar.
     */
    function &loadPartial(&$access, $extended)
    {
        global $conf;

        $file = $this->_getFilename($access->folder, $access->owner);
        $pvCal = $this->getPartials($access)->loadPartialVcal($file, $extended);
        return $pvCal;
    }

    /**
     * Load the complete free/busy data of a user.
     *
     * @param Horde_Kolab_FreeBusy_Access $access   The object holding the
     *                                        relevant access
     *                                        parameters.
     * @param boolean               $extended Should the data hold the extended
     *                                        free/busy information?
     *
     * @return Horde_iCalendar|PEAR_Error The free/busy data for a user.
     */
    function load($access, $extended)
    {
        require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Combined.php';
        require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Combined/Decorator/Cache.php';
        $combined = new Horde_Kolab_FreeBusy_Export_Freebusy_Combined_Decorator_Cache(
            new Horde_Kolab_FreeBusy_Export_Freebusy_Combined(
                $access, $this->getPartials($access)
            ),
            $access,
            $this->_cache_dir
        );

        return $combined->generate($extended);
    }

    /**
     * Get a cache file name depending on the owner of the free/busy
     * data.
     *
     * @param string  $folder  Name of the calendar folder.
     * @param string  $owner   Owner of the calendar folder.
     *
     * @return string Name of the correspoding cache file.
     */
    function _getFilename($folder, $owner)
    {
        if (preg_match('/(.*)@(.*)/', $owner, $regs)) {
            $owner = $regs[2] . '/' . $regs[1];
        }

        return str_replace("\0", '', str_replace('.', '^', $owner . '/' . $folder));
    }
}