<?php
/**
 * A cache file for ACLs. This serves as a buffer between the DB based
 * ACL storage and is required to hold the old ACL list for updates to
 * the DB based cache.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * A cache file for ACLs. This serves as a buffer between the DB based
 * ACL storage and is required to hold the old ACL list for updates to
 * the DB based cache.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache/File/acl.php,v 1.1.2.2 2010/06/15 12:00:34 wrobel Exp $
 *
 * Copyright 2004-2008 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_File_acl extends Horde_Kolab_FreeBusy_Cache_File {

    /**
     * The suffix of this cache file.
     *
     * @var string
     */
    var $_suffix = 'acl';

    /**
     * Link to the ACL stored in a data base.
     *
     * @var Horde_Kolab_FreeBusy_Cache_DB
     */
    var $_acls;

    /**
     * Construct the Horde_Kolab_FreeBusy_Cache_File_acl instance.
     *
     * @param string $cache_dir The path to the cache direcory.
     */
    function Horde_Kolab_FreeBusy_Cache_File_acl($cache_dir)
    {
        $this->_acls = &Horde_Kolab_FreeBusy_Cache_DB::singleton('acl', $cache_dir);
        parent::Horde_Kolab_FreeBusy_Cache_File($cache_dir, 'acl');
    }

    /**
     * Clean the cache file contents.
     *
     * @param string $filename The file name of the cache file.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function purge($filename)
    {
        $this->setFilename($filename);

        $oldacl = $this->load();
        if (is_a($oldacl, 'PEAR_Error')) {
            $oldacl = array();
        }

        $result = $this->_acls->store($this->_filename, array(), $oldacl, false);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return parent::purge();
    }

    /**
     * Store a new ACL.
     *
     * @param string $filename  The file name of the cache file.
     * @param array  $acl       The new ACL.
     * @param string $relevance Folder relevance.
     * @param string $append    Should old entries be purged?
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function storeACL($filename, &$acl, $relevance, $append = false)
    {
        $this->setFilename($filename);

        if (!$append) {
            $oldacl = $this->load();
            if (is_a($oldacl, 'PEAR_Error')) {
                $oldacl = array();
            }
            $acl = array_merge($oldacl, $acl);
        } else {
            $oldacl = array();
        }

        /* Handle relevance */
        switch ($relevance) {
        case 'readers':
            $perm = 'r';
            break;
        case 'nobody':
            $perm = false;
            break;
        case 'admins':
        default:
            $perm = 'a';
        }

        $result = $this->_acls->store($this->_filename, $acl, $oldacl, $perm);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $result = $this->store($acl);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        return true;
    }
}
