<?php
/**
 * A cache file for extended ACLs. This serves as a buffer between the
 * DB based ACL storage and is required to hold the old ACL list for
 * updates to the DB based cache.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * A cache file for extended ACLs. This serves as a buffer between the
 * DB based ACL storage and is required to hold the old ACL list for
 * updates to the DB based cache.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache/File/xacl.php,v 1.1.2.2 2010/06/15 12:00:34 wrobel Exp $
 *
 * Copyright 2004-2008 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_File_xacl extends Horde_Kolab_FreeBusy_Cache_File {

    /**
     * The suffix of this cache file.
     *
     * @var string
     */
    var $_suffix = 'xacl';

    /**
     * Link to the ACL stored in a data base.
     *
     * @var Horde_Kolab_FreeBusy_Cache_DB
     */
    var $_xacls;

    /**
     * Construct the Horde_Kolab_FreeBusy_Cache_File_xacl instance.
     *
     * @param string $cache_dir The path to the cache direcory.
     */
    function Horde_Kolab_FreeBusy_Cache_File_xacl($cache_dir)
    {
        $this->_xacls = &Horde_Kolab_FreeBusy_Cache_DB::singleton('xacl', $cache_dir);
        parent::Horde_Kolab_FreeBusy_Cache_File($cache_dir, 'xacl');
    }

    /**
     * Clean the cache file contents.
     *
     * @param string $filename The file name of the cache file.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function purge($filename)
    {
        $this->setFilename($filename);

        $oldxacl = $this->load();
        if (is_a($oldxacl, 'PEAR_Error')) {
            $oldxacl = '';
        }

        $result = $this->_xacls->store($this->_filename, '', $oldxacl);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return parent::purge();
    }

    /**
     * Store a new extended ACL.
     *
     * @param string $filename The file name of the cache file.
     * @param array  $xacl     The new extended ACL.
     * @param array  $acl      General ACL for the folder.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function storeXACL($filename, &$xacl, &$acl)
    {
        $this->setFilename($filename);

        $oldxacl = $this->load();
        if (is_a($oldxacl, 'PEAR_Error')) {
            $oldxacl = '';
        }

        /* Users with read access to the folder may also access the extended information */
        foreach ($acl as $user => $ac) {
            if (strpos($ac, 'r') !== false) {
                if (!empty($user)) {
                    $xacl .= ' ' . $user;
                }
            }
        }

        $result = $this->_xacls->store($this->_filename, $xacl, $oldxacl);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $result = $this->store($xacl);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        return true;
    }
}
