<?php
/**
 * Handles a cached partial free/busy list.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Handles a cached partial free/busy list.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you did not
 * receive this file, see
 * http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_Freebusy_Partial
{
    /**
     * The cache handler for the partial vCalendar file.
     *
     * @var Horde_Kolab_FreeBusy_Cache_File_pvcal
     */
    private $_partial_cache;

    /**
     * The vcalendar information loaded from the cache.
     *
     * @var Horde_iCalendar
     */
    private $_vcal;

    public function __construct(
        Horde_Kolab_FreeBusy_Cache_File_pvcal $partial_cache
    ) {
        $this->_partial_cache = $partial_cache;
    }

    /**
     * Get the path to the cache file.
     *
     * @return string The path to the file.
     */
    function getFilename()
    {
        return $this->_partial_cache->getFilename();
    }

    public function load($extended)
    {
        $this->_vcal = $this->_partial_cache->loadPVcal($extended);
        return $this->_vcal;
    }

    public function purge()
    {
        $this->_vcal = null;
        return $this->_partial_cache->purge();
    }

    public function store($vCal)
    {
        $result = $this->_partial_cache->storePVcal($vCal);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        $this->_vcal = $result;
    }

    public function isValidVCalendar()
    {
        if (!empty($this->_vcal)) {
            return new PEAR_Error('Load the cache file first!');
        }
        if (!is_a($this->_vcal, 'PEAR_Error')) {
            return true;
        }
        return $this->_vcal;
    }

    public function getFreeBusy()
    {
        if (empty($this->_vcal)) {
            return false;
        }
        if (is_a($this->_vcal, 'PEAR_Error')) {
            return $this->_vcal;
        }
        return $this->_vcal->findComponent('vfreebusy');
    }

    public function isOutdated()
    {
        if ($this->getFreeBusy() === false) {
            return;
        }
        if ($this->getFreeBusyEnd() !== false
            && $this->getFreeBusyEnd() < time()) {
            $this->_partial_cache->purge();
            return true;
        }
        return false;
    }

    private function getFreeBusyEnd()
    {
        $fb = $this->getFreeBusy();
        if (is_a($fb, 'PEAR_Error')) {
            return $fb;
        }
        return $fb->getAttributeDefault('DTEND', false);
    }

    public function getValidatedFreeBusy($extended_pvc)
    {
        $this->load($extended_pvc);
        if (is_a($this->isValidVCalendar(), 'PEAR_Error')
            || !$this->getFreeBusy()
            || $this->isOutdated()) {
            return false;
        }
        return $this->getFreeBusy();
    }
}