<?php
/**
 * Logs access to a cached partial free/busy list.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Logs access to a cached partial free/busy list.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you did not
 * receive this file, see
 * http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_Freebusy_Partial_Decorator_Log
{
    /**
     * The observed partial.
     *
     * @var Horde_Kolab_FreeBusy_Cache_Freebusy_Partial
     */
    private $_partial;

    public function __construct(
        Horde_Kolab_FreeBusy_Cache_Freebusy_Partial $partial
    ) {
        $this->_partial = $partial;
    }

    /**
     * Get the path to the cache file.
     *
     * @return string The path to the file.
     */
    function getFilename()
    {
        return $this->_partial->getFilename();
    }

    public function load($extended)
    {
        return $this->_partial->load($extended);
    }

    public function purge()
    {
        $result = $this->_partial->purge();
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage(sprintf("Failed purging cache file %s: %s)",
                                      $this->_partial->getFilename(), $result->getMessage()),
                              __FILE__, __LINE__, PEAR_LOG_INFO);
        } else {
            Horde::logMessage(sprintf("No events. Purging cache %s.",
                                      $this->_partial->getFilename()),
                              __FILE__, __LINE__, PEAR_LOG_DEBUG);
        }
        return $result;
    }

    public function store($vCal)
    {
        $result = $this->_partial->store($vCal);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage(sprintf("Failed storing cache file %s: %s)",
                                      $this->_partial->getFilename(), $result->getMessage()),
                              __FILE__, __LINE__, PEAR_LOG_INFO);
        } else {
            Horde::logMessage(sprintf("Stored vCalendar cache file %s.",
                                      $this->_partial->getFilename()),
                              __FILE__, __LINE__, PEAR_LOG_DEBUG);
        }
        return $result;
    }

    public function isValidVCalendar()
    {
        $result = $this->_partial->isValidVCalendar();
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage(sprintf("Invalid partial free/busy file %s: %s)",
                                      $this->_partial->getFilename(), $result->getMessage()),
                              __FILE__, __LINE__, PEAR_LOG_INFO);
        }
        return $result;
    }

    public function getFreeBusy()
    {
        if (!$this->_partial->getFreeBusy()) {
            Horde::logMessage(sprintf("Could not find free/busy info in file %s.)",
                                      $this->getFilname()), __FILE__, __LINE__, PEAR_LOG_INFO);
        }
        return $this->_partial->getFreeBusy();
    }

    public function isOutdated()
    {
        if ($this->_partial->isOutdated() === true) {
            Horde::logMessage(sprintf("Free/busy info in file %s is too old.)",
                                      $this->getFilename()), __FILE__, __LINE__, PEAR_LOG_INFO);
        }
        return $this->_partial->isOutdated();
    }

    public function getValidatedFreeBusy($extended_pvc)
    {
        $this->load($extended_pvc);
        if (!$this->isValidVCalendar()
            || !$this->getFreeBusy()
            || $this->isOutdated()) {
            return false;
        }
        return $this->getFreeBusy();
    }
}