<?php
/**
 * Handles the cached partial free/busy lists.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

require_once 'Horde/Kolab/FreeBusy/Cache/Freebusy/Partial.php';
require_once 'Horde/Kolab/FreeBusy/Cache/Freebusy/Partial/Decorator/Log.php';

/**
 * Handles the cached partial free/busy lists.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you did not
 * receive this file, see
 * http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_Freebusy_Partials
{
    /**
     * The directory used for caching.
     *
     * @var string
     */
    private $_cache_dir;

    /**
     * ACL handler.
     *
     * @var Horde_Kolab_FreeBusy_Export_Freebusy_Acl
     */
    private $_acl_access;

    /**
     * Extended ACL handler.
     *
     * @var Horde_Kolab_FreeBusy_Export_Freebusy_Xacl
     */
    private $_xacl_access;

    /**
     * Constructor.
     *
     * @param string                                    $cache_dir   Path to the cache directory.
     * @param Horde_Kolab_FreeBusy_Export_Freebusy_Acl  $acl_access  ACL access handler.
     * @param Horde_Kolab_FreeBusy_Export_Freebusy_Xacl $xacl_access Extended ACL access handler.
     */
    public function __construct(
        $cache_dir,
        Horde_Kolab_FreeBusy_Export_Freebusy_Acl $acl_access,
        Horde_Kolab_FreeBusy_Export_Freebusy_Xacl $xacl_access
    ) {
        $this->_cache_dir = $cache_dir;
        $this->_acl_access  = $acl_access;
        $this->_xacl_access = $xacl_access;
    }

    private function getPartial($file)
    {
        $cache = new Horde_Kolab_FreeBusy_Cache_File_pvcal(
            $this->_cache_dir
        );
        $cache->setFilename($file);
        return new Horde_Kolab_FreeBusy_Cache_Freebusy_Partial_Decorator_Log(
            new Horde_Kolab_FreeBusy_Cache_Freebusy_Partial(
                $cache
            )
        );
    }

    public function loadPartialFreeBusy($file, $extended)
    {
        return $this->getPartial($file)->getValidatedFreeBusy($extended);
    }

    public function loadPartialVcal($file, $extended)
    {
        if ($extended) {
            $extended = $this->allowExtended($file);
        }
        return $this->getPartial($file)->load($extended);
    }

    public function purgePartialVcal($file)
    {
        $result = $this->getPartial($file)->purge();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        $result = $this->_acl_access->purge($file);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        return $this->_xacl_access->purge($file);
    }

    public function storePartialVcal($file, $vCal, $fb)
    {
        $result = $this->getPartial($file)->store($vCal);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        $result = $this->_acl_access->store($file, $fb);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        return $this->_xacl_access->store($file, $fb, $result);
    }

    public function merge($vfb, $file, $extended)
    {
        if (!$this->loadPartialFreeBusy($file, $extended)) {
            return $vfb;
        }
        $vfb->merge($this->loadPartialFreeBusy($file, $extended));
        return $vfb;
    }

    public function getMtimes($file)
    {
        $c_pvcal = new Horde_Kolab_FreeBusy_Cache_File_pvcal($this->_cache_dir, $file);
        return array($c_pvcal->getFile(), $c_pvcal->getMtime());
    }

    public function getFiles()
    {
        return $this->_acl_access->getFiles();
    }

    public function allowExtended($file)
    {
        return $this->_xacl_access->allow($file);
    }
}