<?php
/**
 * Test combining partial free/busy scripts into a single vCalendar.
 *
 * PHP version 5
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Prepare the test setup.
 */
require_once dirname(__FILE__) . '/../../../../../Autoload.php';

require_once 'Horde/Kolab/FreeBusy/Access.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB/xacl.php';
require_once 'Horde/Kolab/FreeBusy/Cache/Freebusy/Partials.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Combined.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl/Cache.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Cache.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Configuration.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Decorator/Log.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Combined/Decorator/Cache.php';

/**
 * Test combining partial free/busy scripts into a single vCalendar.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Class_Export_Freebusy_Combined_Decorator_CacheTest
extends PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        global $conf;

        $this->cache_dir = Horde::getTempDir() . '/Kolab_FreeBusy_Test_' . md5(uniqid(mt_rand(), true));

        $this->access = $this->getMock('Horde_Kolab_FreeBusy_Access');
        $this->xaclcache = new Stub_Xacl(array('test' => array('group@example.org')));
        $this->aclcache = $this->getMock('Horde_Kolab_FreeBusy_Cache_DB_acl', array('get'), array(), '', false, false);
        $this->xaclfilecache = $this->getMock('Horde_Kolab_FreeBusy_Cache_File_xacl', array(), array(), '', false, false);
        $this->aclfilecache = $this->getMock('Horde_Kolab_FreeBusy_Cache_File_acl', array(), array(), '', false, false);
        $this->access->user_object = new Stub_User(array());
        $this->access->user = 'anonymous@example.org';

        $this->access->owner_object = $this->getMock('Horde_Kolab_Server_Object', array('get'), array(), '', false, false);
        $this->access->owner_object->expects($this->any())
            ->method('get')
            ->with(KOLAB_ATTR_CN)
            ->will($this->returnValue('Mr. Test'));
        $this->access->owner = 'test@example.org';

        $this->xaclaccess_cache = new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Decorator_Log(
            new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Cache(
                $this->access, $this->xaclcache, $this->xaclfilecache
            )
        );

        $this->aclaccess_cache = new Horde_Kolab_FreeBusy_Export_Freebusy_Acl_Cache(
            $this->access, $this->aclcache, $this->aclfilecache
        );

        $this->xaclaccess_configuration = new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Decorator_Log(
            new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Configuration(
                $this->access, true
            )
        );

        $this->partials = new Stub_Partials($this->cache_dir, $this->aclaccess_cache, $this->xaclaccess_cache);

        $this->combined = new Horde_Kolab_FreeBusy_Export_Freebusy_Combined(
            $this->access, $this->partials
        );

        $this->cache = new Horde_Kolab_FreeBusy_Export_Freebusy_Combined_Decorator_Cache(
            $this->combined, $this->access, $this->cache_dir
        );

        $conf = array();
        $conf['log']['enabled'] = true;
        $conf['log']['type'] = 'null';
        $conf['log']['name'] = 'mock';
        $conf['log']['ident'] = 'mock';
        $conf['log']['params'] = array();
        $conf['log']['priority'] = PEAR_LOG_DEBUG;
        $log = Horde::getLogger();
        $this->observer = new Log_Observer_Mock(PEAR_LOG_DEBUG);
        $log->attach($this->observer);
    }

    public function testMethodGetsignature()
    {
        global $conf;

        $conf['fb']['vcal_cache']['min_age'] = -1;

        $this->aclcache->expects($this->any())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array('test')));

        $vfb = $this->cache->generate(true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );

        $this->access->user_object = new Stub_User(array('group@example.org'));

        $vfb = $this->cache->generate(true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );
    }
}
