<?php
/**
 * Test combining partial free/busy scripts into a single vCalendar.
 *
 * PHP version 5
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Prepare the test setup.
 */
require_once dirname(__FILE__) . '/../../../Autoload.php';

require_once 'Horde/Kolab/FreeBusy/Access.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB.php';
require_once 'Horde/Kolab/FreeBusy/Cache/DB/xacl.php';
require_once 'Horde/Kolab/FreeBusy/Cache/File.php';
require_once 'Horde/Kolab/FreeBusy/Cache/File/pvcal.php';
require_once 'Horde/Kolab/FreeBusy/Cache/Freebusy/Partials.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Combined.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Acl/Cache.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Cache.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Configuration.php';
require_once 'Horde/Kolab/FreeBusy/Export/Freebusy/Xacl/Decorator/Log.php';

/**
 * Test combining partial free/busy scripts into a single vCalendar.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Class_Export_Freebusy_CombinedTest
extends PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        global $conf;

        $this->access = $this->getMock('Horde_Kolab_FreeBusy_Access');
        $this->xaclcache = $this->getMock('Horde_Kolab_FreeBusy_Cache_DB_xacl', array(), array(), '', false, false);
        $this->aclcache = $this->getMock('Horde_Kolab_FreeBusy_Cache_DB_acl', array('get'), array(), '', false, false);
        $this->xaclfilecache = $this->getMock('Horde_Kolab_FreeBusy_Cache_File_xacl', array(), array(), '', false, false);
        $this->aclfilecache = $this->getMock('Horde_Kolab_FreeBusy_Cache_File_acl', array(), array(), '', false, false);
        $this->access->user_object = $this->getMock('Horde_Kolab_Server_Object_user', array('getGroupAddresses'), array(), '', false, false);
        $this->access->user = 'anonymous@example.org';
        $this->access->user_object->expects($this->any())
            ->method('getGroupAddresses')
            ->will($this->returnValue(array()));

        $this->access->owner_object = $this->getMock('Horde_Kolab_Server_Object', array('get'), array(), '', false, false);
        $this->access->owner_object->expects($this->any())
            ->method('get')
            ->with(KOLAB_ATTR_CN)
            ->will($this->returnValue('Mr. Test'));
        $this->access->owner = 'test@example.org';

        $this->xaclaccess_cache = new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Decorator_Log(
            new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Cache(
                $this->access, $this->xaclcache, $this->xaclfilecache
            )
        );

        $this->aclaccess_cache = new Horde_Kolab_FreeBusy_Export_Freebusy_Acl_Cache(
            $this->access, $this->aclcache, $this->aclfilecache
        );

        $this->xaclaccess_configuration = new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Decorator_Log(
            new Horde_Kolab_FreeBusy_Export_Freebusy_Xacl_Configuration(
                $this->access, true
            )
        );

        $this->partials = new Stub_Partials('', $this->aclaccess_cache, $this->xaclaccess_cache);

        $this->combined = new Horde_Kolab_FreeBusy_Export_Freebusy_Combined(
            $this->access, $this->partials
        );

        $conf = array();
        $conf['log']['enabled'] = true;
        $conf['log']['type'] = 'null';
        $conf['log']['name'] = 'mock';
        $conf['log']['ident'] = 'mock';
        $conf['log']['params'] = array();
        $conf['log']['priority'] = PEAR_LOG_DEBUG;
        $log = Horde::getLogger();
        $this->observer = new Log_Observer_Mock(PEAR_LOG_DEBUG);
        $log->attach($this->observer);
    }

    public function testMethodGenerateDeliversEmptyVcalendarIfNoPartialsWereGiven()
    {
        $this->aclcache->expects($this->once())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array()));

        $result = $this->combined->generate();
        $result = $result[0]->exportvCalendar();
        $vcal = new Horde_iCalendar();
        $result = $vcal->parsevCalendar($result);
        $vfb = $vcal->findComponent('vfreebusy');

        $this->assertEquals(
            'This is a dummy vfreebusy that indicates an empty calendar',
            $vfb->getAttribute('COMMENT')
        );
    }

    public function testMethodGenerateLoadsExtendedPartialDataIfRequestedAndAllowed()
    {
        $this->aclcache->expects($this->any())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array('test')));

        $this->xaclcache->expects($this->once())
            ->method('has')
            ->with('test', 'anonymous@example.org')
            ->will($this->returnValue(true));

        $this->combined->generate(true);
    }

    public function testMethodGenerateDoesNotLoadExtendedPartialDataIfRequestedAndDisallowed()
    {
        $this->aclcache->expects($this->any())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array('test')));

        $this->xaclcache->expects($this->once())
            ->method('has')
            ->with('test', 'anonymous@example.org')
            ->will($this->returnValue(false));

        $this->combined->generate(true);
    }

    public function testMethodGenerateLogsExtendedAclResultIfExtendedDataWasRequestedAndCurrentUserIsAnonymous()
    {
        $this->aclcache->expects($this->any())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array('test')));

        $this->access->user_object = null;

        $this->combined->generate(true);

        $this->assertContains(
            'Extended attributes on file test disallowed for user "anonymous".',
            $this->observer->events[0]['message']
        );
    }

    public function testMethodGenerateLogsExtendedAclResultIfExtendedDataWasRequestedAndDenied()
    {
        $this->aclcache->expects($this->any())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array('test')));

        $this->xaclcache->expects($this->once())
            ->method('has')
            ->with('test', 'anonymous@example.org')
            ->will($this->returnValue(false));

        $this->combined->generate(true);

        $this->assertContains(
            'Extended attributes on file test disallowed for user "anonymous@example.org".',
            $this->observer->events[0]['message']
        );
    }

    public function testMethodGetsignature()
    {
        $this->aclcache->expects($this->any())
            ->method('get')
            ->with('test@example.org')
            ->will($this->returnValue(array('test')));

        $this->xaclcache->expects($this->once())
            ->method('has')
            ->with('test', 'anonymous@example.org')
            ->will($this->returnValue(true));

        $this->assertEquals(
            '3d32c86e0a0c19ecd9da963732365dd9',
            $this->combined->getSignature(true)
        );
    }
}
